@@ -28,6 +28,7 @@ extern off_t total_size;
 
 extern int cvs_exclude;
 
+extern int one_file_system;
 extern int make_backups;
 extern int preserve_links;
 extern int preserve_perms;
@@ -49,6 +50,17 @@ static int match_file_name(char *fname,struct stat *st)
   return 1;
 }
 
+/* used by the one_file_system code */
+static dev_t filesystem_dev;
+
+static void set_filesystem(char *fname)
+{
+  struct stat st;
+  if (lstat(fname,&st) != 0) return;
+  filesystem_dev = st.st_dev;
+}
+
+
 static void send_directory(int f,struct file_list *flist,char *dir);
 
 static char *flist_dir = NULL;
@@ -104,6 +116,9 @@ static struct file_struct *make_file(int recurse,char *fname)
     return NULL;
   }
 
+  if (one_file_system && st.st_dev != filesystem_dev)
+    return NULL;
+
   if (!match_file_name(fname,&st))
     return NULL;
 
@@ -149,6 +164,7 @@ static struct file_struct *make_file(int recurse,char *fname)
 }
 
 
+
 static void send_file_name(int f,struct file_list *flist,
 			   int recurse,char *fname)
 {
@@ -156,7 +172,7 @@ static void send_file_name(int f,struct file_list *flist,
 
   file = make_file(recurse,fname);
 
-  if (!file) return;
+  if (!file) return;  
   
   if (flist->count >= flist->malloced) {
     flist->malloced += 100;
@@ -167,9 +183,10 @@ static void send_file_name(int f,struct file_list *flist,
       out_of_memory("send_file_name");
   }
 
-  flist->files[flist->count++] = *file;    
-
-  send_file_entry(file,f);
+  if (strcmp(file->name,".") && strcmp(file->name,"/")) {
+    flist->files[flist->count++] = *file;    
+    send_file_entry(file,f);
+  }
 
   if (S_ISDIR(file->mode) && recurse) {
     char **last_exclude_list = local_exclude_list;
@@ -281,6 +298,8 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
 	continue;
       }
       flist_dir = dir;
+      if (one_file_system)
+	set_filesystem(fname);
       send_file_name(f,flist,recurse,fname);
       flist_dir = NULL;
       if (chdir(dbuf) != 0) {
@@ -290,6 +309,8 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
       continue;
     }
 
+    if (one_file_system)
+      set_filesystem(fname);
     send_file_name(f,flist,recurse,fname);
   }
 
