@@ -40,6 +40,7 @@ int dry_run=0;
 int local_server=0;
 int ignore_times=0;
 int delete_mode=0;
+int one_file_system=0;
 
 int am_server = 0;
 static int sender = 0;
@@ -121,6 +122,8 @@ static void server_options(char **args,int *argc)
     argstr[x++] = 'C';
   if (ignore_times)
     argstr[x++] = 'I';
+  if (one_file_system)
+    argstr[x++] = 'x';
   argstr[x] = 0;
 
   if (x != 1) args[ac++] = argstr;
@@ -300,6 +303,9 @@ void do_server_recv(int argc,char *argv[])
     }    
   }
 
+  if (delete_mode)
+    recv_exclude_list(STDIN_FILENO);
+
   flist = recv_file_list(STDIN_FILENO);
   if (!flist || flist->count == 0) {
     fprintf(stderr,"nothing to do\n");
@@ -348,6 +354,7 @@ static void usage(FILE *f)
   fprintf(f,"-D, --devices            preserve devices (root only)\n");
   fprintf(f,"-t, --times              preserve times\n");  
   fprintf(f,"-n, --dry-run            show what would have been transferred\n");
+  fprintf(f,"-x, --one-file-system    don't cross filesystem boundaries\n");
   fprintf(f,"-B, --block-size SIZE    checksum blocking size\n");  
   fprintf(f,"-e, --rsh COMMAND        specify rsh replacement\n");
   fprintf(f,"-C, --cvs-exclude        auto ignore files in the same way CVS does\n");
@@ -366,7 +373,7 @@ static void usage(FILE *f)
 enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,
       OPT_EXCLUDE_FROM,OPT_DELETE};
 
-static char *short_options = "oblpguDCtcahvrIne:B:";
+static char *short_options = "oblpguDCtcahvrIxne:B:";
 
 static struct option long_options[] = {
   {"version",     0,     0,    OPT_VERSION},
@@ -375,6 +382,7 @@ static struct option long_options[] = {
   {"delete",      0,     0,    OPT_DELETE},
   {"exclude",     1,     0,    OPT_EXCLUDE},
   {"exclude-from",1,     0,    OPT_EXCLUDE_FROM},
+  {"one-file-system",0,  0,    'x'},
   {"ignore-times",0,     0,    'I'},
   {"help",        0,     0,    'h'},
   {"dry-run",     0,     0,    'n'},
@@ -430,6 +438,10 @@ int main(int argc,char *argv[])
 	  ignore_times = 1;
 	  break;
 
+	case 'x':
+	  one_file_system=1;
+	  break;
+
 	case OPT_DELETE:
 	  delete_mode = 1;
 	  break;
@@ -655,6 +667,8 @@ int main(int argc,char *argv[])
     if (sender) {
       if (cvs_exclude)
 	add_cvs_excludes();
+      if (delete_mode) 
+	send_exclude_list(f_out);
       flist = send_file_list(f_out,recurse,argc,argv);
       if (verbose > 3) 
 	fprintf(stderr,"file list sent\n");
